"""
Indexbinder

A very small library to help you store and retrieve faiss indexes from cloud storage.
"""

__version__ = "0.1.1"

from .indexing import BaseIndexer
from .image_search import ImageSearch
